/**
  *  \file planetscentral/net/mimebuilder.h
  *  \brief Networking: MIME builder
  */
#ifndef C2SERVER_PLANETSCENTRAL_NET_MIMEBUILDER_H
#define C2SERVER_PLANETSCENTRAL_NET_MIMEBUILDER_H

#include <list>
#include <iosfwd>
#include "cpluslib/string.h"

/** Simple MIME Multipart Entity Builder. This gathers up a MIME
    message line by line, and fills in the boundaries.

    Usage:
    - construct an object <tt>MimeBuilder b("content-type");</tt>
      giving it the multipart content type as parameter
    - use addHeader(...) to add additional headers
    - use addBoundary() followed by multiple addHeader(), followed
      by addLine()/addBase64()/addLineQP() to add parts
    - use addBoundary() to add the final boundary
    - use finish() to finish the object
    - possibly use removeInitialHeaders()
    - use computeSize() and write() to get the data

    Content-Transfer-Encoding is not managed by MimeBuilder, you must
    write a Content-Transfer-Encoding line that matches your addXxx()
    calls yourself. */

class MimeBuilder {
    /** Type of an element. */
    enum ElementType {
        TextElement,              /**< Raw text. */
        ContentTypeElement,       /**< Content-type line. */
        BoundaryElement           /**< Boundary. */
    };
    /** Element (line) of our message. */
    struct Element {
        string_t m_Text;
        ElementType m_Type;

        Element(const string_t& text, ElementType type)
            : m_Text(text), m_Type(type) { }
        
    };
    /** Content of our message. */
    std::list<Element> m_Content;

    /** Content-Type to use. */
    string_t m_ContentType;

    /** Accumulator for building base64 parts. */
    string_t m_Base64Accumulator;

    /** Set if we have a pending CRLF after binary data. */
    bool m_PendingCRLF;

    /** State. */
    enum State {
        StateHeader,
        StateBody
    };
    State m_State;

    void flushHeaderAndStartBody();
    void flushBase64();
 public:
    MimeBuilder(string_t ctype);

    ~MimeBuilder();

    void addHeader(string_t name, string_t value);

    void addLine(string_t line);

    void addBase64(const char* data, std::size_t size);

    void addLineQP(string_t line);

    void addRawData(const char* data, std::size_t size);

    void addBoundary();

    void finish();

    std::size_t computeSize() const;

    void write(std::ostream& out, bool escapeDots);

    string_t removeInitialHeaders();
};

#endif
