/**
  *  \file planetscentral/talk/forum.cc
  */

#include <time.h>
#include "planetscentral/talk/forum.h"
#include "planetscentral/talk/group.h"
#include "planetscentral/talk/db.h"
#include "int/array.h"
#include "cpluslib/misc.h"
#include "planetscentral/talk/render.h"

Forum::Forum(DbClient& client, int32_t forumId)
    : forum(DbSubtree(client, FORUM_ROOT).subtree(forumId)),
      forumId(forumId)
{ }

DbHash
Forum::header()
{
    return forum.hashKey("header");
}

DbHashStringField
Forum::name()
{
    return header().stringField("name");
}

DbHashStringField
Forum::description()
{
    return header().stringField("description");
}

DbHashStringField
Forum::readPermissions()
{
    return header().stringField("readperm");
}

DbHashStringField
Forum::writePermissions()
{
    return header().stringField("writeperm");
}

DbHashStringField
Forum::answerPermissions()
{
    return header().stringField("answerperm");
}


DbHashStringField
Forum::deletePermissions()
{
    return header().stringField("deleteperm");
}

DbHashStringField
Forum::key()
{
    return header().stringField("key");
}

DbHashIntField
Forum::lastMessageSequenceNumber()
{
    return header().intField("msgseq");
}

DbHashIntField
Forum::creationTime()
{
    return header().intField("time");
}

DbHashIntField
Forum::lastPost()
{
    return header().intField("lastpost");
}

DbHashIntField
Forum::lastTime()
{
    return header().intField("lasttime");
}


DbIntSet
Forum::messages()
{
    return forum.intSetKey("messages");
}

DbIntSet
Forum::topics()
{
    return forum.intSetKey("threads");
}

DbIntSet
Forum::stickyTopics()
{
    return forum.intSetKey("stickythreads");
}

DbStringSet
Forum::watchers()
{
    return forum.stringSetKey("watchers");
}

void
Forum::setParent(string_t newParent)
{
    string_t oldParent = getParent();
    Group oldGroup(forum.getClient(), oldParent);
    Group newGroup(forum.getClient(), newParent);
    if (oldParent != newParent) {
        if (oldParent.empty()) {
            newGroup.forums().add(forumId);
        } else if (newParent.empty()) {
            oldGroup.forums().remove(forumId);
        } else {
            oldGroup.forums().moveTo(forumId, newGroup.forums());
        }
        header().setField("parent", newParent);
    }
}

string_t
Forum::getParent()
{
    return header().getStringField("parent");
}

void
Forum::setNewsgroup(string_t newNG)
{
    string_t oldNG(getNewsgroup());
    DbHash ngMap(DbSubtree(forum.getClient(), FORUM_ROOT).hashKey("newsgroups"));

    if (oldNG != newNG) {
        // If this newsgroup name is already taken, drop it from there
        int32_t oldForum(ngMap.getIntField(newNG));
        if (oldForum != 0) {
            Forum(forum.getClient(), oldForum).header().removeField("newsgroup");
        }

        // Also remove our own old name
        if (!oldNG.empty()) {
            ngMap.removeField(oldNG);
        }

        // Update
        ngMap.setField(newNG, forumId);
        header().setField("newsgroup", newNG);
    }
}

string_t
Forum::getNewsgroup()
{
    return header().getStringField("newsgroup");
}

int32_t
Forum::getId() const
{
    return forumId;
}

int32_t
Forum::allocateForum(DbClient& client)
{
    return ++DbSubtree(client, FORUM_ROOT).intKey("id");
}

bool
Forum::exists()
{
    return DbSubtree(forum.getClient(), FORUM_ROOT).intSetKey("all").contains(forumId);
}

IntValue*
Forum::describe(RenderState& state)
{
    // FIXME: can we use HMGET?
    const char*const fields[] = { "name", "parent", "description", "newsgroup" };
    Ptr<IntArrayData> ad = new IntArrayData();
    for (std::size_t i = 0; i < countof(fields); ++i) {
        ad->content.pushNew(makeStringValue(fields[i]));
        if (fields[i] == "description") {
            ad->content.pushNew(makeStringValue(state.render(description().get(), forum.getClient())));
        } else {
            ad->content.pushNew(header().getFieldRaw(fields[i]));
        }
    }
    ad->addDimension(ad->content.size());
    return new IntArray(ad);
}

/** Get time. There is no need to configure a time scale as for c2host, as we're
    not going to configure this to run faster somehow; c2talk has no dependencies
    on wall-clock times. But, as for c2host, let's store minutes only. This makes
    the values appearing on the network interface the same as for c2host, and will
    survive 2038 :-) */
int32_t
getTime()
{
    return time(0) / 60;
}
